#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2006 Nikos Kouremenos

import sys
import os
import signal

import MySQLdb
import cElementTree as ElementTree

class Database:
	def __init__(self):
		pass

	def create_db_and_table(self):
		self.con = MySQLdb.connect('localhost', USERNAME, PASSWORD)
		self.cur = self.con.cursor()
		# create db
		print 'create db'
		self.cur.execute('CREATE DATABASE diglib DEFAULT CHARACTER SET utf8')
		print 'use db'
		self.cur.execute('USE diglib')
		# create the table and the index
		print 'create table and fulltext indeces'
		self.cur.execute(
			'''
			CREATE TABLE courses(
				row_id INTEGER PRIMARY KEY AUTO_INCREMENT UNIQUE,
				course_id INTEGER UNIQUE,
				title TEXT,
				creator TEXT,
				educationLevel TEXT,
				description TEXT,
				tableOfContents TEXT,
				created INTEGER,
				modified INTEGER,
				subject TEXT,
				publisher TEXT,
				identifier TEXT,
				
				FULLTEXT (title, description, tableOfContents, subject),
				FULLTEXT (title, description),
				FULLTEXT (title, tableOfContents),
				FULLTEXT (title, subject)
			)
			DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci;
			'''
			)
		
		self.cur.execute('SET character_set_results="utf8"')
		self.con.commit()

	def ensure_utf8_string(self, text):
		if text is not None:
			text = text.encode('utf-8')
		return text

	def add_values_to_db(self, values_per_course, course_id):
		title = self.ensure_utf8_string(values_per_course['title'])
		creator = self.ensure_utf8_string(values_per_course['creator'])
		educationLevel = self.ensure_utf8_string(
			values_per_course['educationLevel'])
		description = self.ensure_utf8_string(values_per_course['description'])
		tableOfContents = self.ensure_utf8_string(
			values_per_course['tableOfContents'])
		created = self.ensure_utf8_string(values_per_course['created'])
		modified = self.ensure_utf8_string(values_per_course['modified'])
		subject = self.ensure_utf8_string(values_per_course['subject'])
		publisher = self.ensure_utf8_string(values_per_course['publisher'])
		identifier = self.ensure_utf8_string(values_per_course['identifier'])
		print 'inserting', title
		self.cur.execute('''
			INSERT INTO courses (course_id, title, creator, educationLevel,
				description, tableOfContents, created, modified, subject, publisher,
				identifier)
			VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)''',
			(course_id, title, creator, educationLevel, description,
				tableOfContents, created, modified, subject, publisher, identifier))

	def fill_db(self):
		'''parses XML and calls add_values_to_db to add the data to DB'''
		values_per_course = {}
		for event, elem in ElementTree.iterparse('lib.xml'):
			pos = elem.tag.find('}') + 1
			tag = elem.tag[pos:]

			if tag == 'course':
				# the course id we are leaving
				course_id = elem.attrib.items()[0][1]
				self.add_values_to_db(values_per_course, course_id)
			elif tag == 'library': # we finished parsing
				break
			else:
				text = elem.text
				if text is not None:
					text = text.strip()
				values_per_course[tag] = text

			elem.clear()

		self.con.commit()

	def diglib_db_exists(self):
		con = MySQLdb.connect('localhost', USERNAME, PASSWORD)
		cur = con.cursor()
		cur.execute('SHOW DATABASES;')
		results =  cur.fetchall()
		for res in results:
			if 'diglib' in res[0]:
				return True

		return False

	def make_db_from_xml(self):
		if self.diglib_db_exists():
			# delete db
			con = MySQLdb.connect('localhost', USERNAME, PASSWORD)
			cur = con.cursor()
			print 'DELETE diglib database'
			cur.execute('DROP DATABASE diglib;')
		
		self.create_db_and_table()
		print 'fill db'
		self.fill_db()

if __name__ == '__main__':
	signal.signal(signal.SIGINT, signal.SIG_DFL) # ^C exits the application
	if len(sys.argv) == 3:
		USERNAME = sys.argv[1]
		PASSWORD = sys.argv[2]
	else:
		print 'Usage:', sys.argv[0], 'username', 'password'
		sys.exit()
	db = Database()
	db.make_db_from_xml()

	print 'Nothing else to do. Bye'
